/************************************************************************
 *   Sumoline_2.c
 *   Copyright 2006 Solarbotics Ltd.
 *
 *  Loosely based on code written by Bob Cook for the ATMega8 Brainboard
 *  add-on kit, derived from original code by Grant McKee.
 *	Cleaned up and ported to the PIC16F877a by Dan Gates 2006
 *
 *	Written for the Rev 2.0 Sumovore PIC16F877a Brainboard Add-on Kit.
 *	Intended to be compiled with PICC Lite via MPLAB IDE and downloaded via
 *	serial port using PIC downloader 1.08 at 38400 bps.
 *  http://www.microchipc.com/PIC16bootload/PIC16F87xA_bootloader_v9-50.zip
 *
 *  Solarbotics: http://www.solarbotics.com/
 *
 *  Description: Minisumo/Linefollower PIC program Ver 2.0
 *
 *  If outside edge sensors see black during startup the program will branch
 *  to sumo. If outside edge sensors see white during startup the program will
 *  branch to Linefollower.
 *
 *  Sumo mode is fairly basic:
 *
 *       - Wait 5 seconds before moving
 *       - After 5 seconds go straight forward
 *       - If a white line is detected on outside Left sensor
 *           * Reverse both motors briefly
 *           * Forward left motor
 *           * Reverse Right motor
 *           * After set time continue straight forward
 *       - If a white line is detected on outside Right sensor
 *           * Reverse both motors briefly
 *           * Forward Right motor
 *           * Reverse Left motor
 *           * After set time continue straight forward
 *       - If opponent detected on right Side
 *           * Stop Right motor
 *       - If opponent detected on Left Side
 *           * Stop Left motor
 *       - If both sensors detect opponent
 *           * Turn on both motors forward
 *
 *   Linefollower mode:
 *
 *       - Start immediately
 *       - If center sensor sees black all is well- go straight forward
 *       - If center right sensor sees black, make a gentle right turn by
 *         slowing down right motor
 *       - If center left sensor sees black, make a gentle left turn by slowing
 *         down left motor
 *       - If far right sensor sees black, make a very sharp right turn by
 *         reversing right motor
 *       - If far left sensor sees black, make a very sharp left turn by
 *         reversing left motor
 *       - If all is white, all is lost - Stop all action!
 *
 ****************************************************************************/

/***************************************************************************
 *
 * Include files
 *
 ***************************************************************************/
#include <htc.h>
#include "sumovore2.h"


/***************************************************************************
 *
 *  Definitions
 *
 ***************************************************************************/
 // Most defines are located in the sumovore2.h include file

 // White line sensitivity; higher is more sensitive (255 max)
#define threshold 127;


/***************************************************************************
 *
 * Functions
 *
 ***************************************************************************/
void     SUMO( void );
void	 delay_1ms(unsigned char msec);
void	 respond(unsigned char thresholdd);
unsigned char adc(unsigned char channel);


/***********************************************************************************
 *  ADCIN
 *
 *  Read the specified Analog to Digital Conversion channel
 *
 ***********************************************************************************/
unsigned char adc( unsigned char channel )
{
    unsigned char i, analog;

    ADCON0 = channel;      		// f877_adcon0 = channel -- Channel select and sets ADON = on
    i=0;
    while( i<=15 ) i++;    		// delay 20 usec
    ADGO = on;          		// adcon0_go = high -- Start Conversion
    while( ADGO == on );    	// while adcon0_go loop end loop -- wait for A/D completion
    analog = ADRESH;            // analog = f877_adresh -- Store the resulting number in analog
    ADON = off;           		// adcon0_go = low -- Stop the ADC

    i=0;
    while( i<=15) i++;          // delay 20 usec to allow channel switch
    return analog;				// return result
}


/***************************************************************************************
 *
 * Delay milisecond
 *
 ***************************************************************************************/
void delay_1ms(unsigned char msec)
{
    unsigned char h, i, j;
    for( j=0; j<msec; j++)       // repeat delay msec times
    {
        for( i=0; i<15; i++ )    // delay 1 ms (0.25 ms four times)
        {                        // used MPLAB simulation to get delay correct
            for(h=0; h<207; h++);
        }
    }
}


/*************************************************************************************
 *
 *  main (program entry point)
 *
 *************************************************************************************/
void main ( void )
{ 	 
	unsigned char thresholdd = threshold;
	 // All of our line sensors are on porta, so we need to set these lines as inputs.
	TRISA = all_inputs;

	 // Portb is not used in this program, so we want to set all lines as outputs so
	 // there is no chance for false inputs to the processor.
	TRISB = all_outputs;

	 // Motor enables and motor direction control are on portc, so we need to set
	 //  these lines as outputs. Then we need to make sure the motors are off and
	 //  the direction set to forward so they're ready to go.
	TRISC = portc_dir;

    enableRmotor = no;
    enableLmotor = no;
    RmotorForw = yes;
    LmotorForw = yes;

     // The Sumovore PIC add-on board has five red leds on portd that can be used
	 // to display things in our program; we need to set these lines as outputs, 
	 //  other lines on this port are inputs.
    TRISD = portd_dir;						// PD0 - PD4 = red leds 1 thru 5

     // make sure all LEDs are off until we're ready.
    PORTD = 0b11111111;

     // Porte is also not being used in this program, so we will set these lines as
     //  outputs like we did on portb.
    TRISE = all_outputs;

     // Enable the Analog to Digital Conversion (ADC)
    ADCON1 = ACDCON1setup; 

	if (( adc(EdgeLeft) > 127 ) && ( adc(EdgeRight) > 127 ))
	{
		SUMO();			// Jump into Mini Sumo mode
	}
	else  //LineFollower
	{
	  	while(1)
	   	{
	     	respond(thresholdd);
	   	}
	}
}
void respond(unsigned char thresholdd)
{

	led1 = ( adc(EdgeLeft) < thresholdd );      // adc() prototye in sumovore.h
	led2 = ( adc(EdgeCntLeft) < thresholdd );   // ledx turns on when corresponding
	led3 = ( adc(EdgeCenter) < thresholdd );	// reflective sensore sees a line
	led4 = ( adc(EdgeCntRight) < thresholdd );  
	led5 = ( adc(EdgeRight) < thresholdd );
	enableLmotor = yes;			// motors on
	enableRmotor = yes;
	LmotorForw = yes;			// motors fwd
	RmotorForw = yes;

 	if ( !led1 )
		{
 		  	LmotorForw = no;  					// right motor forward
		}

	else if ( !led2 )
		{
	  		enableLmotor = no;			     	// right motor enable
		}

	else if ( !led3 )
		{
			enableLmotor = yes;			// motors on
			enableRmotor = yes;
			LmotorForw = yes;			// motors fwd
			RmotorForw = yes;
		}

	else if ( !led4 )
		{
			enableRmotor = no;
		}

	else if ( !led5 )
		{
			RmotorForw = no;
		}

	else  // I'm Lost!!
		{
			enableLmotor = no;			// motors on
			enableRmotor = no;
		}
			

}


/**********************************************************************
 *  
 *  SUMO Mode
 *  Find and push the fat diapered japanese guy out of the ring!
 *
 **********************************************************************/
void SUMO()
{
	PORTD = 0b11111111;				// Make sure all LEDs are off

	// Five second countdown

	led1 = ledon;
 	delay_1ms(1000);
	led2 = ledon;
	delay_1ms(1000);
	led3 = ledon;
 	delay_1ms(1000);
	led4 = ledon;
 	delay_1ms(1000);
	led5 = ledon;
	delay_1ms(1000);

    // Loop here forever, chasing our opponent out of the ring

  for(;;)
    {
		PORTD = 0b11111111;				// Turn all LEDs aff again
	
		// Turn on both motors going full forward
	
	  	enableLmotor = yes;     			// left  motor enable
	  	enableRmotor = yes;			     	// right motor enable
	  	LmotorForw = yes;    				// left  motor forward
		RmotorForw = yes;  					// right motor forward

        // read the line sensors to see if we've hit the ring edge

        if ( adc(EdgeRight) < 127 )
        {
			// Turn on Right LED

			led5 = ledon;

            // Right side sees ring edge; reverse both motors and turn around

		  	LmotorForw = no;    				// left  motor reverse
			RmotorForw = no;  					// right motor reverse
		  	delay_1ms( 400 );

			RmotorForw = yes;  					// right motor forward
		  	delay_1ms( 400 );
        }

        if ( adc(EdgeLeft) < 127 )
        {
			// Turn Left LED on

			led1 = ledon;

            // Right side sees ring edge; reverse both motors and turn around

		  	LmotorForw = no;    				// left  motor reverse
			RmotorForw = no;  					// right motor reverse
		  	delay_1ms( 400 );

			LmotorForw = yes;  					// left motor forward
		  	delay_1ms( 400 );
        }

        // Read the forward-looking sensors to find our opponent

		if ( RightIR && LeftIR )	// Fat japanese guy straight ahead, Get Him!
		{
			led2 = ledon;					// turn on LED 2
			led4 = ledon;					// turn on LED 4
			delay_1ms(10);
		}

		else if ( RightIR )			// Fat japanese guy spotted to the right
		{
			led2 = ledoff;					// turn off LED 2
			led4 = ledon;					// turn on LED 4
	
		  	enableRmotor = no;				// right motor enable
			delay_1ms(100);
		}

		else if ( LeftIR )			// Fat japanese guy spotted to the left
		{
			led2 = ledon;					// turn on LED 2
			led4 = ledoff;					// turn off LED 4
	
		  	enableLmotor = no;				// left  motor enable
			delay_1ms(100);
		}
	}
}
